package cn.fw.dalaran.rpc.erp;

import cn.fw.dalaran.rpc.erp.dto.ShopMsg;
import cn.fw.dalaran.rpc.erp.dto.UserInfoDTO;
import cn.fw.data.base.domain.common.Message;
import cn.fw.erp.sdk.api.UserRoleApi;
import cn.fw.erp.sdk.api.result.UserRoleDataRange;
import cn.fw.erp.sdk.api.result.UserRoleInfo;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 用户角色RPC服务
 * <p>
 * create at 2020-06-15
 *
 * @author kurisu
 */
@Slf4j
@Service
public class UserRoleRpcService {

    private final UserRoleApi userRoleApi;

    @Autowired
    public UserRoleRpcService(UserRoleApi userRoleApi) {
        this.userRoleApi = userRoleApi;
    }

    /**
     * 查询指定服务站指定角色的用户信息
     *
     * @param shopId   服务站id
     * @param roleCode 角色代码
     */
    public List<UserInfoDTO> getUsers(Long shopId, String roleCode) {
        if (Objects.isNull(shopId) || Objects.isNull(roleCode)) {
            return Collections.emptyList();
        }
        String func = "调用ERP系统查询指定服务站指定角色的用户信息";
        try {
            Message<List<UserRoleInfo>> msg = userRoleApi.queryUserRoleInfos(roleCode, shopId);
            log.info("{}，shopId：{}，roleCode：{}，result：{}", func, shopId, roleCode, JSON.toJSONString(msg));
            if (msg.isSuccess()) {
                final List<UserRoleInfo> returnData = msg.getData();
                if (CollectionUtils.isEmpty(returnData)) {
                    return Collections.emptyList();
                }
                List<UserInfoDTO> list = new ArrayList<>();
                for (UserRoleInfo roleInfo : returnData) {
                    UserInfoDTO dto = new UserInfoDTO();
                    BeanUtils.copyProperties(roleInfo, dto);
                    list.add(dto);
                }
                return list;
            }
        } catch (Exception e) {
            log.error("{}失败，shopId：{}，roleCode：{}", func, shopId, roleCode, e);
        }
        return Collections.emptyList();
    }

    /**
     * 查询用户角色编码对应授权门店
     *
     * @param userId   用户id
     * @param roleCode 角色编码
     */
    public List<ShopMsg> getUserRoleRange(Long userId, String roleCode) {
        try {
            final Message<List<UserRoleDataRange>> msg = userRoleApi.queryUserRoleAllRange(userId, roleCode);
            log.info("调用ERP系统查询人员角色授权门店, userId：{}, roleCode：{}, result：{}", userId, roleCode, JSON.toJSONString(msg));
            if (msg.isSuccess()) {
                final List<UserRoleDataRange> returnData = msg.getData();
                if (CollectionUtils.isEmpty(returnData)) {
                    return Collections.emptyList();
                }
                return returnData.stream()
                        .map(item -> {
                            final ShopMsg shopMsg = new ShopMsg();
                            shopMsg.setShopId(item.getRangeValue());
                            shopMsg.setShopName(item.getRangeName());
                            return shopMsg;
                        }).collect(Collectors.toList());
            }
        } catch (Exception e) {
            log.error("调用ERP系统查询人员角色授权门店失败，userId：{}，roleCode：{}", userId, roleCode, e);
        }
        return Collections.emptyList();
    }
}
